﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ResxCrunch.Utils.FileData
{
    /// <summary>
    /// Csv file info data
    /// </summary>
    public class CsvData
    {
        #region Attributes
        /// <summary>
        /// Indicates whether subdirectories should be scanned
        /// </summary>
        private bool includeHeaders = true;
        #endregion Attributes

        #region Initialization
        /// <summary>
        /// Initializes a new instance of the CsvData class
        /// </summary>
        /// <param name="path">The full path to the CSV file, including the filename</param>
        /// <param name="includeHeaders">Indicates if headers will be included into the file or only data</param>
        /// <param name="separatorIndex">Index of the separator.</param>
        public CsvData(string path, bool includeHeaders, int separatorIndex)
        {
            this.Path = path;
            this.IncludeHeaders = includeHeaders;
            this.SeparatorIndex = separatorIndex;
        }
        #endregion Initialization

        #region Properties
        /// <summary>
        /// Gets or sets path to the CSV file
        /// </summary>
        public string Path { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether subdirectories should be scanned
        /// </summary>
        public bool IncludeHeaders
        {
            get
            {
                return includeHeaders;
            }

            set
            {
                includeHeaders = value;
            }
        }

        /// <summary>
        /// Gets or sets the separator index.
        /// </summary>
        /// <value>The separator index</value>
        public int SeparatorIndex { get; set; }
        #endregion Properties
    }
}
